package io.skore.video360kaltura

import android.content.Context
import android.view.View
import android.widget.FrameLayout
import com.kaltura.playkit.*
import com.kaltura.playkit.player.vr.VRInteractionMode
import com.kaltura.playkit.player.vr.VRSettings
import com.kaltura.playkit.providers.ovp.OVPMediaAsset
import com.kaltura.playkitvr.VRController
import com.kaltura.playkitvr.VRUtil
import com.kaltura.tvplayer.KalturaOvpPlayer
import com.kaltura.tvplayer.OVPMediaOptions
import com.kaltura.tvplayer.PlayerInitOptions
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class VideoPlayerController(private val context: Context?, private val listener: ViewCreatedListener, val viewId: Int, val binaryMessenger: BinaryMessenger): MethodChannel.MethodCallHandler {
    private var entryId: String? = null
    private var partnerId: Int? = null
    private var ovpServerUrl: String? = null

    private var player: KalturaOvpPlayer? = null
    private var mediaEntry: PKMediaEntry? = null
    private var videoPlayerState: VideoPlayerState? = null
    private var isVRModeEnabled: Boolean = false

    private var playerEventStateChanged: EventChannel.EventSink? = null
    private var playerEventDurationChanged: EventChannel.EventSink? = null
    private var playerEventPosition: EventChannel.EventSink? = null
    private var playerEventEnded: EventChannel.EventSink? = null

    interface ViewCreatedListener {
        fun onViewCreated(view: View)
        fun changeViewSize(args: HashMap<*, *>)
    }

    init {
        MethodChannel(binaryMessenger, "video360Kaltura_$viewId").apply {
            setMethodCallHandler(this@VideoPlayerController)
        }
        createEventChannel("video360Kaltura_events_${viewId}_state", { e -> playerEventStateChanged = e}, { playerEventStateChanged = null })
        createEventChannel("video360Kaltura_events_${viewId}_duration", { e -> playerEventDurationChanged = e}, { playerEventDurationChanged = null })
        createEventChannel("video360Kaltura_events_${viewId}_position", { e -> playerEventPosition = e}, { playerEventPosition = null })
        createEventChannel("video360Kaltura_events_${viewId}_ended", { e -> playerEventEnded = e}, { playerEventEnded = null })
    }

    private fun createEventChannel(name: String, onListen: (events: EventChannel.EventSink?) -> Unit, onCancel: () -> Unit) {
        EventChannel(binaryMessenger, name).setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                onListen(events)
            }

            override fun onCancel(arguments: Any?) {
                onCancel()
            }
        })
    }

    companion object {
        private const val TAG = "VideoControllerKaltura"
    }

    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {
        when (methodCall.method) {
            "loadVideo" -> {
                ((methodCall.arguments as? HashMap<*, *>)?.get("entryId") as? String)?.let {
                    loadMedia(it, result)
                }
            }
            "seekTo" -> {
                (methodCall.arguments as? HashMap<*, *>)?.get("position")?.toString()?.toLongOrNull()?.let { position ->
                    player?.let {
                        if (position < it.duration) {
                            it.seekTo(position)
                            result.success(methodCall.arguments)
                        }
                    }
                }
            }
            "play" -> {
                player?.play()
                result.success(true)
            }
            "pause" -> {
                player?.pause()
                result.success(true)
            }
            "isPlaying" -> {
                result.success(player?.isPlaying == true)
            }
            "onSizeChanged" -> {
                if (mediaEntry?.isVRMediaType == true) {
                    reloadPlayer()
                }

                (methodCall.arguments as? HashMap<*, *>)?.let {
                    listener.changeViewSize(it)
                }
                result.success(true)
            }
            "toggleVRMode" -> {
                toggleVRMode()
            }
            "fullScreen" -> {
                if (mediaEntry?.isVRMediaType == true) {
                    dispose()
                }
                result.success(true)
            }
            "onPause" -> {
                if (mediaEntry?.isVRMediaType == true) {
                    dispose()
                } else {
                    player?.onApplicationPaused()
                }
                result.success(true)
            }
            "onResume" -> {
                if (mediaEntry?.isVRMediaType == true) {
                    reloadPlayer()
                } else {
                    player?.onApplicationResumed()
                }
                result.success(true)
            }
            "onOrientationChanged" -> {
                if (mediaEntry?.isVRMediaType == true) {
                    dispose()
                }
                result.success(true)
            }
            else -> result.notImplemented()
        }
    }

    private fun reloadPlayer() {
        loadPlayer()
        this.entryId?.let {
            loadMedia(it, null)
        }
    }

    private fun toggleVRMode() {
        player?.getController(VRController::class.java)?.let { vr ->
            isVRModeEnabled = !isVRModeEnabled
            vr.enableVRMode(isVRModeEnabled)
        } ?: run {
            isVRModeEnabled = !isVRModeEnabled
        }
    }

    private fun loadMedia(entryId: String, result: MethodChannel.Result?) {
        this.entryId = entryId
        val ovpMediaOptions = buildOvpMediaOptions(entryId)
        player?.loadMedia(ovpMediaOptions) { entry, loadError ->
            if (loadError == null) {
                this.mediaEntry = entry.apply {
                    result?.success(true)
                }
            }
        }
    }

    private fun buildOvpMediaOptions(entryId: String): OVPMediaOptions {
        val ovpMediaAsset = OVPMediaAsset()
        ovpMediaAsset.entryId = entryId
        ovpMediaAsset.ks = null
        val ovpMediaOptions = OVPMediaOptions(ovpMediaAsset)
        ovpMediaOptions.startPosition = 0
        return ovpMediaOptions
    }

    fun loadPlayer(partnerId: Int, ovpServerUrl: String) {
        this.partnerId = partnerId
        this.ovpServerUrl = ovpServerUrl
        loadPlayer()
    }

    private fun loadPlayer() {
        if (player != null) return

        val playerInitOptions = PlayerInitOptions(this.partnerId)
        playerInitOptions.setAutoPlay(false)
        playerInitOptions.setVRSettings(configureVRSettings())

        player = KalturaOvpPlayer.create(context, playerInitOptions).apply {
            setPlayerView(FrameLayout.LayoutParams.WRAP_CONTENT, FrameLayout.LayoutParams.WRAP_CONTENT)
            listener.onViewCreated(playerView)

            addListener(this)
        }
    }

    private fun addListener(kalturaOvpPlayer: KalturaOvpPlayer) {
        kalturaOvpPlayer.addListener<PlayerEvent.StateChanged>(this, PlayerEvent.stateChanged) { event ->
            when (event.newState) {
                PlayerState.READY -> {
                    videoPlayerState = videoPlayerState?.let {
                        kalturaOvpPlayer.seekTo(it.currentPosition)
                        isVRModeEnabled = true
                        toggleVRMode()
                        if (it.isPlaying) kalturaOvpPlayer.play()
                        null
                    }
                    playerEventStateChanged?.success(mapOf(Pair("state", 1)))
                }
                PlayerState.LOADING -> {
                    playerEventStateChanged?.success(mapOf(Pair("state", 0)))
                }
                PlayerState.BUFFERING -> {
                    playerEventStateChanged?.success(mapOf(Pair("state", 2)))
                }
            }
        }

        kalturaOvpPlayer.addListener<PlayerEvent.PlayheadUpdated>(this, PlayerEvent.playheadUpdated) { progressUpdate ->
            playerEventPosition?.success(mapOf(Pair("currentPosition", progressUpdate.position)))
        }

        kalturaOvpPlayer.addListener<PlayerEvent.DurationChanged>(this, PlayerEvent.durationChanged) { mediaDuration ->
            playerEventDurationChanged?.success(mapOf(Pair("duration", mediaDuration.duration)))
        }

        kalturaOvpPlayer.addListener(this, PlayerEvent.ended) {
            playerEventEnded?.success(mapOf(Pair("ended", true)))
        }
    }

    private fun configureVRSettings(): VRSettings {
        val vrSettings = VRSettings()
        vrSettings.isFlingEnabled = true
        vrSettings.isVrModeEnabled = false
        vrSettings.interactionMode = VRInteractionMode.CardboardMotionWithTouch
        vrSettings.isZoomWithPinchEnabled = true

        val interactionMode = vrSettings.interactionMode
        if (!VRUtil.isModeSupported(context, interactionMode)) {
            vrSettings.interactionMode = VRInteractionMode.Touch
        }
        return vrSettings
    }

    fun dispose() {
        player?.let {
             this.videoPlayerState = VideoPlayerState(it.currentPosition, it.isPlaying)
            it.destroy()
        }
        player = null
    }

    inner class VideoPlayerState(val currentPosition: Long = 0, val isPlaying: Boolean = false)
}